
/* ************************************************************************* */
/*
 *	sha256.c
 *
 *	(C) 2010 VOCAL Technologies, Ltd.
 *
 *	This software is intended for use as a reference/standard implementation,
 *	and has been placed into the public domain by the copyright holder:
 *
 *	VOCAL Technologies, Ltd.
 *	90A John Muir Drive.
 *	Buffalo, NY  14228
 *
 *	Maintainer:  Dennis Towne
 */
/* ************************************************************************* */


#include <errno.h>
#include <stdio.h>
#include <string.h>

#include "system.h"
#include "util.h"
#include "sha2.h"

void usage(void)
{
	fprintf(stderr, "Usage: sha256 [filenames]\n");
	fprintf(stderr, "\n");
	fprintf(stderr, "This program calculates the sha-256 hash of a given input.\n");
	fprintf(stderr, "Input comes from standard in.\n");
}

int sha256_hash_file(FILE *stream, uchar hash[SHA256dlen]) {
	SHA256state st;
	uchar buf[32*1024];
	int n;

	sha256init(&st);
	for(;;) {
		n = fread(buf, 1, sizeof(buf), stream);
		if (n < 0) {
			/* fprintf(stderr, "read: %s\n", strerror(errno)); */
			return 0;
		}
		if(n == 0) {
			break;
		}
		sha256(&st, buf, (unsigned int)n);
	}
	sha256finish(&st, hash);
	return 1;
}

int main(int argc, char *argv[])
{
	int n;
	uchar hash[SHA256dlen];
	char shash[2*SHA256dlen+1];

	if (argc <= 1) {
		sha256_hash_file(stdin, hash);	/* get input from standard in fd 0 */

		hex(hash, SHA256dlen, shash);
		printf("%s\n", shash);
		return 0;
	}

	if (argv[1] != NULL && (strcmp(argv[1], "-h") == 0 || strcmp(argv[1], "--help") == 0)) {
		usage();
		return 1;
	}
	
	for(n = 1; n < argc; n++) {
		FILE *stream = fopen(argv[n], "rb");
		if (stream == NULL) {
			printf("Unable to open file '%s'.\n", argv[n]);
		}
		else {
			if (!sha256_hash_file(stream, hash)) {
				printf("Error reading from file '%s'.\n", argv[n]);
			}
			else {
				hex(hash, SHA256dlen, shash);
				printf("%s: %s\n", argv[n], shash);
			}
			fclose(stream);
		}
	}

	return 0;
}
